var __create = Object.create;
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __getProtoOf = Object.getPrototypeOf;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toESM = (mod, isNodeMode, target) => (target = mod != null ? __create(__getProtoOf(mod)) : {}, __copyProps(isNodeMode || !mod || !mod.__esModule ? __defProp(target, "default", { value: mod, enumerable: true }) : target, mod));
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);

// src/Ajax.ts
var Ajax_exports = {};
__export(Ajax_exports, {
  default: () => Ajax
});
module.exports = __toCommonJS(Ajax_exports);
var import_axios = __toESM(require("axios"));
var import_qs = require("qs");
var import_utils = require("./utils");
var Ajax = class {
  constructor(options = {}) {
    const {
      onSuccess = () => void 0,
      onError = () => void 0,
      noEmpty = true,
      undefinedToNull = false,
      trim = true,
      fillUrl = true,
      fixU202D = true,
      spaceUnicodeToASCII = true,
      ...defaults
    } = options;
    this.instance = import_axios.default.create();
    this.instance.defaults.timeout = 1e3 * 60;
    this.instance.defaults.headers["Content-Type"] = "application/json;charset=UTF-8";
    this.instance.defaults.baseURL = "/";
    (0, import_utils.setDefaults)(defaults, this.instance);
    this.onSuccess = onSuccess;
    this.onError = onError;
    this.noEmpty = noEmpty;
    this.undefinedToNull = undefinedToNull;
    this.trim = trim;
    this.fillUrl = fillUrl;
    this.fixU202D = fixU202D;
    this.spaceUnicodeToASCII = spaceUnicodeToASCII;
  }
  ajax(options = {}) {
    let {
      noEmpty = this.noEmpty,
      undefinedToNull = this.undefinedToNull,
      trim = this.trim,
      fillUrl = this.fillUrl,
      fixU202D = this.fixU202D,
      spaceUnicodeToASCII = this.spaceUnicodeToASCII,
      originResponse = false,
      successTip = false,
      errorTip,
      cancelRef = (c) => c,
      setLoading = () => void 0,
      setDeleting = () => void 0,
      setSaving = () => void 0,
      setUpdating = () => void 0,
      setFetching = () => void 0,
      setUploading = () => void 0,
      setDownloading = () => void 0,
      setExporting = () => void 0,
      params,
      data,
      url,
      method = "get",
      ...otherOptions
    } = options;
    const _setLoading = (loading) => [
      setLoading,
      setDeleting,
      setUpdating,
      setSaving,
      setFetching,
      setUploading,
      setDownloading,
      setExporting
    ].forEach((fn) => (0, import_utils.setPublicLoading)(fn, loading));
    if (trim) {
      data = (0, import_utils.trimObject)(data);
      params = (0, import_utils.trimObject)(params);
    }
    if (noEmpty) {
      data = (0, import_utils.emptyObject)(data);
      params = (0, import_utils.emptyObject)(params);
    }
    if (undefinedToNull) {
      data = (0, import_utils.setObjectUndefinedToNull)(data);
      params = (0, import_utils.setObjectUndefinedToNull)(params);
    }
    if (fixU202D) {
      data = (0, import_utils.fixObjectU202D)(data);
      params = (0, import_utils.fixObjectU202D)(params);
    }
    if (spaceUnicodeToASCII) {
      data = (0, import_utils.objectSpaceUnicodeToASCII)(data);
      params = (0, import_utils.objectSpaceUnicodeToASCII)(params);
    }
    const contentType = (0, import_utils.getContentType)(otherOptions.headers) || (0, import_utils.getContentType)(this.instance.defaults.headers);
    const isFormContentType = contentType && contentType.indexOf("application/x-www-form-urlencoded") > -1;
    if (isFormContentType)
      data = data && (0, import_qs.stringify)(data);
    let _url = url;
    if (fillUrl)
      _url = (0, import_utils.setUrlPathParams)(_url, params || data);
    return new Promise((resolve, reject) => {
      _setLoading(true);
      let isCanceled = false;
      const cancelToken = new import_axios.default.CancelToken((c) => {
        cancelRef(() => {
          c("canceled");
          isCanceled = true;
        });
      });
      this.instance({
        method,
        url: _url,
        data,
        params,
        cancelToken,
        ...otherOptions
      }).then((response) => {
        const responseData = originResponse ? response : response.data;
        this.onSuccess({
          data: responseData,
          tip: successTip,
          from: "ajax",
          options,
          response
        });
        resolve(responseData);
      }).catch(async (err) => {
        var _a;
        if (isCanceled)
          return;
        if (((_a = err == null ? void 0 : err.response) == null ? void 0 : _a.data) instanceof Blob) {
          if (["application/json", "text/plain"].includes(err.response.data.type)) {
            err.response.data = await blobToJson(err.response.data);
          }
        }
        this.onError({
          options,
          error: err,
          tip: errorTip,
          from: "ajax"
        });
        return reject(err);
      }).finally(() => {
        _setLoading(false);
      });
    });
  }
  get(url, params, options = {}) {
    return this.ajax({ url, params, method: "get", ...options });
  }
  post(url, data, options = {}) {
    return this.ajax({ url, data, method: "post", ...options });
  }
  put(url, data, options = {}) {
    return this.ajax({ url, data, method: "put", ...options });
  }
  patch(url, data, options = {}) {
    return this.ajax({ url, data, method: "patch", ...options });
  }
  del(url, params, options = {}) {
    return this.ajax({ url, params, method: "delete", ...options });
  }
  upload(url, params, options = {}) {
    const { method = "post", headers = {}, ...otherOptions } = options;
    const contentType = (0, import_utils.getContentType)(headers) || "multipart/form-data;";
    const data = params instanceof FormData ? params : Object.entries(params).reduce((prev, curr) => {
      const [key, value] = curr;
      prev.append(key, value);
      return prev;
    }, new FormData());
    return this.ajax({
      url,
      data,
      method,
      headers: { ...headers, "Content-Type": contentType },
      ...otherOptions
    });
  }
  async download(url, params, options = {}) {
    const {
      fileName,
      method = "get",
      originResponse = true,
      beforeDownload = () => true,
      responseType = "blob",
      errorTip,
      successTip,
      ...others
    } = options;
    let data = {};
    let _params = params;
    if (["patch", "post", "put"].includes(method.toLowerCase())) {
      data = params;
      _params = {};
    }
    const res = await this.ajax({
      method,
      url,
      params: _params,
      data,
      originResponse,
      responseType,
      errorTip,
      successTip: false,
      ...others
    });
    const showError = (err) => {
      if (errorTip !== false) {
        this.onError({
          options,
          error: err,
          tip: errorTip,
          from: "ajax"
        });
      }
      throw err;
    };
    if (beforeDownload(res) === false)
      return;
    const errorMessage = "download fail";
    if (!res || !res.headers || !res.data) {
      return showError(Error(errorMessage));
    }
    if (["application/json", "text/plain"].includes(res.data.type)) {
      try {
        throw await blobToJson(res.data);
      } catch (e) {
        showError(e);
      }
      return;
    }
    const _fileName = fileName || (0, import_utils.getFileName)(res == null ? void 0 : res.headers);
    if (!_fileName)
      return showError(new Error("file name can not be null!"));
    const blob = new Blob([res.data], { type: res.headers["content-type"] });
    const link = document.createElement("a");
    link.setAttribute("href", window.URL.createObjectURL(blob));
    link.setAttribute("download", decodeURIComponent(_fileName || ""));
    link.style.visibility = "hidden";
    document.body.appendChild(link);
    link.click();
    document.body.removeChild(link);
    if (successTip) {
      const responseData = originResponse ? res : res.data;
      this.onSuccess({
        data: responseData,
        tip: successTip,
        from: "ajax",
        options,
        response: res
      });
    }
    return res;
  }
};
function blobToJson(data) {
  return new Promise((resolve, reject) => {
    const reader = new FileReader();
    reader.readAsText(data, "utf-8");
    reader.onload = () => {
      try {
        const resultStr = reader.result;
        const result = JSON.parse(resultStr);
        resolve(result);
      } catch (e) {
        reject(e);
      }
    };
  });
}
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {});
